@@ -1,26 +1,26 @@
-/************************** BEGIN MidiUI.h **************************/
-/************************************************************************
- FAUST Architecture File
- Copyright (C) 2003-2017 GRAME, Centre National de Creation Musicale
- ---------------------------------------------------------------------
- This Architecture section is free software; you can redistribute it
- and/or modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 3 of
- the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; If not, see <http://www.gnu.org/licenses/>.
- 
- EXCEPTION : As a special exception, you may create a larger work
- that contains this FAUST architecture section and distribute
- that work under terms of your choice, so long as this FAUST
- architecture section is not modified.
- ************************************************************************/
+/************************** BEGIN MidiUI.h ****************************
+FAUST Architecture File
+Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
+---------------------------------------------------------------------
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation; either version 2.1 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+EXCEPTION : As a special exception, you may create a larger work
+that contains this FAUST architecture section and distribute
+that work under terms of your choice, so long as this FAUST
+architecture section is not modified.
+************************************************************************/
 
 #ifndef FAUST_MIDIUI_H
 #define FAUST_MIDIUI_H
@@ -28,7 +28,6 @@
 #include <vector>
 #include <string>
 #include <utility>
-#include <iostream>
 #include <cstdlib>
 #include <cmath>
 
@@ -39,7 +38,6 @@
 #include "faust/gui/MapUI.h"
 #include "faust/gui/MetaDataUI.h"
 #include "faust/midi/midi.h"
-#include "faust/gui/MetaDataUI.h"
 #include "faust/gui/ValueConverter.h"
 
 #ifdef _MSC_VER
@@ -48,10 +46,9 @@
 #define gsscanf sscanf
 #endif
 
-/*****************************************************************************
-* Helper code for MIDI meta and polyphonic 'nvoices' parsing
-******************************************************************************/
-
+/**
+ * Helper code for MIDI meta and polyphonic 'nvoices' parsing.
+ */
 struct MidiMeta : public Meta, public std::map<std::string, std::string> {
     
     void declare(const char* key, const char* value)
@@ -111,23 +108,25 @@ struct MidiMeta : public Meta, public std::map<std::string, std::string> {
         for (int i = 0; i < map_ui.getParamsCount(); i++) {
             std::string path = map_ui.getParamAddress(i);
             has_freq |= MapUI::endsWith(path, "/freq");
+            has_freq |= MapUI::endsWith(path, "/key");
             has_gate |= MapUI::endsWith(path, "/gate");
             has_gain |= MapUI::endsWith(path, "/gain");
+            has_gain |= MapUI::endsWith(path, "/vel");
+            has_gain |= MapUI::endsWith(path, "/velocity");
         }
         return (has_freq && has_gate && has_gain);
     }
     
 };
 
-/*******************************************************************************
- * MidiUI : Faust User Interface
+/**
+ * uiMidi : Faust User Interface
  * This class decodes MIDI meta data and maps incoming MIDI messages to them.
- * Currently ctrl, keyon/keyoff, keypress, pgm, chanpress, pitchwheel/pitchbend
+ * Currently ctrlChange, keyOn/keyOff, keyPress, progChange, chanPress, pitchWheel/pitchBend
  * start/stop/clock meta data is handled.
  * MIDI channel is numbered in [1..16] in this layer.
  * Channel 0 means "all channels" when receiving or sending.
- ******************************************************************************/
-
+ */
 class uiMidi {
     
     friend class MidiUI;
@@ -149,10 +148,9 @@ class uiMidi {
 
 };
 
-/*****************************************************************************
- * Base class for MIDI aware UI items
- ******************************************************************************/
-
+/**
+ * Base class for MIDI aware UI items.
+ */
 class uiMidiItem : public uiMidi, public uiItem {
     
     public:
@@ -167,10 +165,9 @@ class uiMidiItem : public uiMidi, public uiItem {
     
 };
 
-/*****************************************************************************
- * Base class for MIDI aware UI items with timestamp support
- ******************************************************************************/
-
+/**
+ * Base class for MIDI aware UI items with timestamp support.
+ */
 class uiMidiTimedItem : public uiMidi, public uiTimedItem {
     
     public:
@@ -185,10 +182,9 @@ class uiMidiTimedItem : public uiMidi, public uiTimedItem {
     
 };
 
-//-----------
-// MIDI sync
-//-----------
-
+/**
+ * MIDI sync.
+ */
 class uiMidiStart : public uiMidiTimedItem
 {
   
@@ -275,14 +271,13 @@ class uiMidiClock : public uiMidiTimedItem {
 
 };
 
-//----------------------
-// Standard MIDI events
-//----------------------
-
-//---------------------------------------------
-// uiMidiProgChange uses the [min...max] range
-//---------------------------------------------
+/**
+ * Standard MIDI events.
+ */
 
+/**
+ * uiMidiProgChange uses the [min...max] range.
+ */
 class uiMidiProgChange : public uiMidiTimedItem {
     
     public:
@@ -329,6 +324,9 @@ class uiMidiProgChange : public uiMidiTimedItem {
         
 };
 
+/**
+ * uiMidiChanPress.
+ */
 class uiMidiChanPress : public uiMidiTimedItem, public uiConverter {
     
     public:
@@ -348,13 +346,14 @@ class uiMidiChanPress : public uiMidiTimedItem, public uiConverter {
         {
             FAUSTFLOAT v = *fZone;
             fCache = v;
+            int conv = std::round(fConverter->faust2ui(v));
             if (fChan == 0) {
                 // Send on [0..15] channels on the MIDI layer
                 for (int chan = 0; chan < 16; chan++) {
-                    fMidiOut->chanPress(chan, fConverter->faust2ui(v));
+                    fMidiOut->chanPress(chan, conv);
                 }
             } else {
-                fMidiOut->chanPress(fChan - 1, fConverter->faust2ui(v));
+                fMidiOut->chanPress(fChan - 1, conv);
             }
         }
     
@@ -374,10 +373,9 @@ class uiMidiChanPress : public uiMidiTimedItem, public uiConverter {
         
 };
 
-//------------------------------------------------------
-// uiMidiCtrlChange does scale (kLin/kLog/kExp) mapping
-//------------------------------------------------------
-
+/**
+ * uiMidiCtrlChange does scale (kLin/kLog/kExp) mapping.
+ */
 class uiMidiCtrlChange : public uiMidiTimedItem, public uiConverter {
     
     private:
@@ -401,13 +399,14 @@ class uiMidiCtrlChange : public uiMidiTimedItem, public uiConverter {
         {
             FAUSTFLOAT v = *fZone;
             fCache = v;
+            int conv = std::round(fConverter->faust2ui(v));
             if (fChan == 0) {
                 // Send on [0..15] channels on the MIDI layer
                 for (int chan = 0; chan < 16; chan++) {
-                    fMidiOut->ctrlChange(chan, fCtrl, fConverter->faust2ui(v));
+                    fMidiOut->ctrlChange(chan, fCtrl, conv);
                 }
             } else {
-                fMidiOut->ctrlChange(fChan - 1, fCtrl, fConverter->faust2ui(v));
+                fMidiOut->ctrlChange(fChan - 1, fCtrl, conv);
             }
         }
         
@@ -426,6 +425,7 @@ class uiMidiCtrlChange : public uiMidiTimedItem, public uiConverter {
         }
 };
 
+// Use a two segments linear converter
 class uiMidiPitchWheel : public uiMidiTimedItem {
 
     private:
@@ -454,13 +454,14 @@ class uiMidiPitchWheel : public uiMidiTimedItem {
         {
             FAUSTFLOAT v = *fZone;
             fCache = v;
+            int conv = std::round(fConverter.faust2ui(v));
             if (fChan == 0) {
                 // Send on [0..15] channels on the MIDI layer
                 for (int chan = 0; chan < 16; chan++) {
-                    fMidiOut->pitchWheel(chan, fConverter.faust2ui(v));
+                    fMidiOut->pitchWheel(chan, conv);
                 }
             } else {
-                fMidiOut->pitchWheel(fChan - 1, fConverter.faust2ui(v));
+                fMidiOut->pitchWheel(fChan - 1, conv);
             }
         }
         
@@ -486,10 +487,9 @@ class uiMidiPitchWheel : public uiMidiTimedItem {
  
 };
 
-//--------------------------------------------------------------
-// uiMidiKeyOn does scale (kLin/kLog/kExp) mapping for velocity
-//--------------------------------------------------------------
-
+/**
+ * uiMidiKeyOn does scale (kLin/kLog/kExp) mapping for velocity.
+ */
 class uiMidiKeyOn : public uiMidiTimedItem, public uiConverter {
 
     private:
@@ -513,13 +513,14 @@ class uiMidiKeyOn : public uiMidiTimedItem, public uiConverter {
         {
             FAUSTFLOAT v = *fZone;
             fCache = v;
+            int conv = std::round(fConverter->faust2ui(v));
             if (fChan == 0) {
                 // Send on [0..15] channels on the MIDI layer
                 for (int chan = 0; chan < 16; chan++) {
-                    fMidiOut->keyOn(chan, fKeyOn, fConverter->faust2ui(v));
+                    fMidiOut->keyOn(chan, fKeyOn, conv);
                 }
             } else {
-                fMidiOut->keyOn(fChan - 1, fKeyOn, fConverter->faust2ui(v));
+                fMidiOut->keyOn(fChan - 1, fKeyOn, conv);
             }
         }
         
@@ -539,10 +540,9 @@ class uiMidiKeyOn : public uiMidiTimedItem, public uiConverter {
     
 };
 
-//---------------------------------------------------------------
-// uiMidiKeyOff does scale (kLin/kLog/kExp) mapping for velocity
-//---------------------------------------------------------------
-
+/**
+ * uiMidiKeyOff does scale (kLin/kLog/kExp) mapping for velocity.
+ */
 class uiMidiKeyOff : public uiMidiTimedItem, public uiConverter {
 
     private:
@@ -566,13 +566,14 @@ class uiMidiKeyOff : public uiMidiTimedItem, public uiConverter {
         {
             FAUSTFLOAT v = *fZone;
             fCache = v;
+            int conv = std::round(fConverter->faust2ui(v));
             if (fChan == 0) {
                 // Send on [0..15] channels on the MIDI layer
                 for (int chan = 0; chan < 16; chan++) {
-                    fMidiOut->keyOn(chan, fKeyOff, fConverter->faust2ui(v));
+                    fMidiOut->keyOff(chan, fKeyOff, conv);
                 }
             } else {
-                fMidiOut->keyOn(fChan - 1, fKeyOff, fConverter->faust2ui(v));
+                fMidiOut->keyOff(fChan - 1, fKeyOff, conv);
             }
         }
         
@@ -592,10 +593,9 @@ class uiMidiKeyOff : public uiMidiTimedItem, public uiConverter {
     
 };
 
-//-----------------------------------------------------------------
-// uiMidiKeyPress does scale (kLin/kLog/kExp) mapping for velocity
-//-----------------------------------------------------------------
-
+/**
+ * uiMidiKeyPress does scale (kLin/kLog/kExp) mapping for velocity.
+ */
 class uiMidiKeyPress : public uiMidiTimedItem, public uiConverter {
 
     private:
@@ -619,13 +619,14 @@ class uiMidiKeyPress : public uiMidiTimedItem, public uiConverter {
         {
             FAUSTFLOAT v = *fZone;
             fCache = v;
+            int conv = std::round(fConverter->faust2ui(v));
             if (fChan == 0) {
                 // Send on [0..15] channels on the MIDI layer
                 for (int chan = 0; chan < 16; chan++) {
-                    fMidiOut->keyOn(chan, fKey, fConverter->faust2ui(v));
+                    fMidiOut->keyPress(chan, fKey, conv);
                 }
             } else {
-                fMidiOut->keyOn(fChan - 1, fKey, fConverter->faust2ui(v));
+                fMidiOut->keyPress(fChan - 1, fKey, conv);
             }
         }
         
@@ -648,7 +649,7 @@ class uiMidiKeyPress : public uiMidiTimedItem, public uiConverter {
 /******************************************************************************************
  * MidiUI : Faust User Interface
  * This class decodes MIDI metadata and maps incoming MIDI messages to them.
- * Currently ctrl, keyon/keyoff, keypress, pgm, chanpress, pitchwheel/pitchbend
+ * Currently ctrlChange, keyOn/keyOff, keyPress, progChange, chanPress, pitchWheel/pitchBend
  * start/stop/clock meta data are handled.
  *
  * Maps associating MIDI event ID (like each ctrl number) with all MIDI aware UI items
@@ -658,7 +659,7 @@ class uiMidiKeyPress : public uiMidiTimedItem, public uiConverter {
  *  - sending their internal state as MIDI output events
  *******************************************************************************************/
 
-class MidiUI : public GUI, public midi, public MetaDataUI {
+class MidiUI : public GUI, public midi, public midi_interface, public MetaDataUI {
 
     // Add uiItem subclasses objects are deallocated by the inherited GUI class
     typedef std::map <int, std::vector<uiMidiCtrlChange*> > TCtrlChangeTable;
@@ -786,13 +787,16 @@ class MidiUI : public GUI, public midi, public MetaDataUI {
         {
             fMidiHandler = midi_handler;
             fMidiHandler->addMidiIn(this);
+            // TODO: use shared_ptr based implementation
             fDelete = delete_handler;
             fTimeStamp = false;
         }
  
         virtual ~MidiUI() 
-        { 
+        {
+            // Remove from fMidiHandler
             fMidiHandler->removeMidiIn(this);
+            // TODO: use shared_ptr based implementation
             if (fDelete) delete fMidiHandler;
         }
     
