@@ -167,6 +167,8 @@ struct Soundfile {
         delete[] fOffset;
     }
 
+    typedef std::vector<std::string> Directories;
+    
 } POST_PACKED_STRUCTURE;
 
 /*
@@ -180,7 +182,7 @@ class SoundfileReader {
     int fDriverSR;
    
     // Check if a soundfile exists and return its real path_name
-    std::string checkFile(const std::vector<std::string>& sound_directories, const std::string& file_name)
+    std::string checkFile(const Soundfile::Directories& sound_directories, const std::string& file_name)
     {
         if (checkFile(file_name)) {
             return file_name;
@@ -265,6 +267,7 @@ class SoundfileReader {
 
   public:
     
+    SoundfileReader() {}
     virtual ~SoundfileReader() {}
     
     void setSampleRate(int sample_rate) { fDriverSR = sample_rate; }
@@ -276,7 +279,7 @@ class SoundfileReader {
             int total_length = 0;
             
             // Compute total length and channels max of all files
-            for (int i = 0; i < int(path_name_list.size()); i++) {
+            for (size_t i = 0; i < path_name_list.size(); i++) {
                 int chan, length;
                 if (path_name_list[i] == "__empty_sound__") {
                     length = BUFFER_SIZE;
@@ -298,7 +301,7 @@ class SoundfileReader {
             int offset = 0;
             
             // Read all files
-            for (int i = 0; i < int(path_name_list.size()); i++) {
+            for (size_t i = 0; i < path_name_list.size(); i++) {
                 if (path_name_list[i] == "__empty_sound__") {
                     soundfile->emptyFile(i, offset);
                 } else {
@@ -307,7 +310,7 @@ class SoundfileReader {
             }
             
             // Complete with empty parts
-            for (int i = int(path_name_list.size()); i < MAX_SOUNDFILE_PARTS; i++) {
+            for (size_t i = path_name_list.size(); i < MAX_SOUNDFILE_PARTS; i++) {
                 soundfile->emptyFile(i, offset);
             }
             
@@ -321,7 +324,7 @@ class SoundfileReader {
     }
 
     // Check if all soundfiles exist and return their real path_name
-    std::vector<std::string> checkFiles(const std::vector<std::string>& sound_directories,
+    std::vector<std::string> checkFiles(const Soundfile::Directories& sound_directories,
                                         const std::vector<std::string>& file_name_list)
     {
         std::vector<std::string> path_name_list;
