@@ -51,6 +51,10 @@ static Plugin* pluginInstance;
 #define FAUSTFLOAT float
 #endif
 
+#ifdef _WIN32
+typedef unsigned int uint
+#endif
+
 /*
 // Pixels are just a weird unit, defined by VCV Rack as 1/75 inches, or 0.338(6) mm
 */
@@ -87,14 +91,22 @@ class rack_dsp {
     
 };
 
+template <typename REAL>
 struct one_sample_dsp : public rack_dsp {
     
     int* iZone;
-    FAUSTFLOAT* fZone;
+    REAL* fZone;
     
-    one_sample_dsp():iZone(nullptr), fZone(nullptr)
+    one_sample_dsp()
+    :iZone(nullptr), fZone(nullptr)
     {}
     
+    one_sample_dsp(int* icontrol, REAL* fcontrol)
+    :iZone(icontrol), fZone(fcontrol)
+    {
+        assert(false);
+    }
+    
     virtual ~one_sample_dsp()
     {
         delete [] iZone;
@@ -110,9 +122,9 @@ struct one_sample_dsp : public rack_dsp {
     virtual int getNumIntControls() = 0;
     virtual int getNumRealControls() = 0;
     
-    virtual void control(int* iControl, FAUSTFLOAT* fControl) = 0;
+    virtual void control(int* iControl, REAL* fControl) = 0;
     
-    virtual void compute(FAUSTFLOAT* inputs, FAUSTFLOAT* outputs, int* iControl, FAUSTFLOAT* fControl) = 0;
+    virtual void compute(FAUSTFLOAT* inputs, FAUSTFLOAT* outputs, int* iControl, REAL* fControl) = 0;
     
     virtual void compute(int count, FAUSTFLOAT** inputs_aux, FAUSTFLOAT** outputs_aux)
     {}
@@ -308,7 +320,7 @@ struct RackUI : public GenericUI
     
     virtual ~RackUI()
     {
-        for (auto& it : fConverters) delete it;
+        for (const auto& it : fConverters) delete it;
     }
     
     void reset()
@@ -484,7 +496,6 @@ struct RackUI : public GenericUI
             fCV = "";
         } else {
             
-            
             if (fScale == "log") {
                 converter = new ConverterZoneControl(zone, new LogValueConverter(0., 1., min, max));
             } else if (fScale == "exp") {
@@ -527,12 +538,12 @@ struct RackUI : public GenericUI
     
     void updateInputs(Module* module)
     {
-        for (auto& it : fUpdateFunIn) it(module);
+        for (const auto& it : fUpdateFunIn) it(module);
     }
     
     void updateOutputs(Module* module)
     {
-        for (auto& it : fUpdateFunOut) it(module);
+        for (const auto& it : fUpdateFunOut) it(module);
     }
 };
 
@@ -719,7 +730,7 @@ struct mydspModule : Module {
         fDSP[0].buildUserInterface(&fLayoutUI);
         
         cout << "==========================" << endl;
-        for (auto& it : fLayoutUI.fPathItemMap) {
+        for (const auto& it : fLayoutUI.fPathItemMap) {
             cout << it.first << endl;
             cout << it.second << endl;
         }
@@ -737,7 +748,7 @@ struct mydspModule : Module {
         fLayoutUI.fCurrentGroup->setPos(x_pos, y_pos);
         
         cout << "==========================" << endl;
-        for (auto& it : fLayoutUI.fPathItemMap) {
+        for (const auto& it : fLayoutUI.fPathItemMap) {
             cout << it.first << endl;
             cout << it.second << endl;
         }
